unit LinkInfo_;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ToolWin, StdCtrls, ExtCtrls, OleCtrls, SHDocVw_TLB,
  DHTMLEventHandler;

type
  TForm1 = class(TForm)
    MouseOut: TDHTMLEventHandler;
    MouseOver: TDHTMLEventHandler;
    IE: TWebBrowser;
    Panel1: TPanel;
    lvStatus: TListView;
    Splitter1: TSplitter;
    Memo1: TMemo;
    ToolBar1: TToolBar;
    btnLoad: TToolButton;
    btnHook: TToolButton;
    btnUnhook: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    edURL: TEdit;
    Label1: TLabel;
    procedure btnLoadClick(Sender: TObject);
    procedure IEDocumentComplete(Sender: TObject; pDisp: IDispatch;
      var URL: OleVariant);
    procedure btnHookClick(Sender: TObject);
    procedure MouseOutEvent(Sender: TDHTMLEventInstance;
      var Done: Boolean);
    procedure MouseOverEvent(Sender: TDHTMLEventInstance;
      var Done: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure btnUnhookClick(Sender: TObject);
    procedure IEBeforeNavigate2(Sender: TObject; pDisp: IDispatch; var URL,
      Flags, TargetFrameName, PostData, Headers: OleVariant;
      var Cancel: WordBool);
  private
    { Private declarations }
    function MakeUpURLInfo(const sURL: string): Variant;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.btnLoadClick(Sender: TObject);
var
  ws: WideString;
  v:  OleVariant;
begin
  ws := edURL.Text;
  IE.Navigate(ws, v, v, v, v);
end;

procedure TForm1.IEDocumentComplete(Sender: TObject; pDisp: IDispatch;
  var URL: OleVariant);
begin
  btnHook.Enabled := True;
  btnUnhook.Enabled := False;
end;

{ This is the key method of the project; in a real browser app, we'd try
  requesting data (or just HTTP header) from the specified URL and collect
  some additional information about it. Here we just pretend to obtain this
  information, actually using some randomly generated data. }
function TForm1.MakeUpURLInfo(const sURL: string): Variant;
var
  ChangeDate, ExpDate: TDateTime;
  Title, MIME: string;
begin
  ChangeDate := Now - (Random(1000) + Random);
  ExpDate    := Now + (Random(1000) + Random);
  Title      := Format('Random Page Title #%d', [Trunc(Random(100))]);
  if Pos(sURL, '.htm') > 0 then
    MIME := 'text/html'
  else if Pos(sURL, '.txt') > 0 then
    MIME := 'text/plain'
  else if Pos(sURL, '.gif') > 0 then
    MIME := 'image/gif';
  Result := VarArrayOf([sURL, Title, ChangeDate, ExpDate, MIME]); 
end;

procedure TForm1.btnHookClick(Sender: TObject);
var
  i: Integer;
  sHREF: string;
  vDoc, vLinks, vLink, vData: Variant;
begin
  vDoc := IE.ControlInterface.Document;
  vLinks := vDoc.links;
  for i := 0 to vLinks.length - 1 do begin
    vLink := vLinks.item(i);
    sHREF := vLink.HREF;
    vData := MakeUpURLInfo(sHREF);
    MouseOver.Add(vLink, 'onmouseover', vData);
    MouseOut.Add(vLink, 'onmouseout', Null);
    Memo1.Lines.Add(Format('#%d. Hooked "%s"', [i+1, sHREF]));
  end;
  btnUnhook.Enabled := True;
  btnHook.Enabled := False;   // can't hook more than once!
  btnLoad.Enabled := False;   // shouldn't reload unless unhooked!
end;

procedure TForm1.MouseOutEvent(Sender: TDHTMLEventInstance;
  var Done: Boolean);
var
  i: Integer;
begin
  for i := 0 to 4 do begin
    lvStatus.Items[i].SubItems[0] := '';
  end;
end;

procedure TForm1.MouseOverEvent(Sender: TDHTMLEventInstance;
  var Done: Boolean);
var
  i: Integer;
begin
  for i := 0 to 4 do begin
    lvStatus.Items[i].SubItems[0] := VarToStr(Sender.Data[i]);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  edURL.Text := ChangeFileExt(Application.ExeName, '.htm');
end;

procedure TForm1.btnUnhookClick(Sender: TObject);
begin
  Memo1.Lines.Add(Format('Unhooking %d links', [MouseOver.InstanceCount]));
  MouseOver.Clear;
  MouseOut.Clear;
  btnHook.Enabled := True;
  btnLoad.Enabled := True;
  btnUnhook.Enabled := False;
end;

procedure TForm1.IEBeforeNavigate2(Sender: TObject; pDisp: IDispatch;
  var URL, Flags, TargetFrameName, PostData, Headers: OleVariant;
  var Cancel: WordBool);
begin
  MouseOver.Clear;
  MouseOut.Clear;
end;

end.
